from threading import Timer
import logging

from watchdog.events import FileSystemEventHandler
from watchdog.observers import Observer

from nginx_site_discovery.extract_sites import walk_sites_enabled
from nginx_site_discovery.prom_sd import create_sd_file


class EventListener(FileSystemEventHandler):
    def __init__(self, watched_dir='.', labels={}, server_url=None, output_file=None, extension='', **kwargs):
        self.labels = labels
        self.watched_dir = watched_dir
        self.output_file = output_file
        self.server_url = server_url
        self.extension = extension
        self.last_urls = set()

        self.timer = Timer(5, self.extract_sites)
        self.timer.start()


    def on_any_event(self, event):
        if self.timer is not None:
            self.timer.cancel()

        self.timer = Timer(5, self.extract_sites)
        self.timer.start()

    def extract_sites(self):
        try:
            urls =  walk_sites_enabled(self.watched_dir, self.extension)

            if urls != self.last_urls:
                logging.info('Detecting change in configuration, creating service discovery file')
                self.last_servers = urls
                create_sd_file(urls, self.labels, self.server_url, self.output_file)
        except Exception as e:
            logging.exception('Error while extracting sites')


def start_watcher(config):
    event_handler = EventListener(**config)
    observer = Observer()
    observer.schedule(event_handler, config['watched_dir'])
    logging.info('Starting watcher')
    return observer
